import logging
from abc import ABC, abstractmethod
from dataclasses import dataclass
from typing import cast, override

import torch.nn as nn
from torch import Tensor
from transformers import PretrainedConfig, PreTrainedModel

from ...config.config_schema import VisualEncoderConfig

log: logging.Logger = logging.getLogger(name=__name__)


@dataclass
class VisualModelConfig:
    hidden_size: int | None = None
    img_size: int | None = None
    patch_size: int | None = None


class VisualEncoder(nn.Module, ABC):
    def __init__(self, config: VisualEncoderConfig) -> None:
        super().__init__()
        self.config: VisualEncoderConfig = config
        self.name: str = self.config.name
        self.hf_name: str = self.config.hf_name
        self.model_type: str = self.config.type

        # model config
        self.model_config: VisualModelConfig = VisualModelConfig(
            hidden_size=getattr(self.config, "hidden_size", None),
            img_size=getattr(self.config, "img_size", None),
            patch_size=getattr(self.config, "patch_size", None),
        )

        # output layer
        self.output_layer: int = getattr(self.config, "output_layer", -1)
        log.info(f"Using {self.output_layer} layer as output layer")

        # use cls token
        self.use_cls_token: bool = getattr(self.config, "use_cls_token", False)
        log.info(f"Use cls token: {self.use_cls_token}")

        self.initialize_components()

    # initialize all components
    def initialize_components(self) -> None:
        self.hf_config: PretrainedConfig = self._build_hf_config()

        self.verify_config()

        self.visual_encoder: PreTrainedModel = self._build_visual_encoder()

        # calculate token size
        self.token_size: int = (
            cast(int, self.model_config.img_size) // cast(int, self.model_config.patch_size)
        ) ** 2 + self.use_cls_token

    @property
    def hidden_size(self) -> int:
        return cast(int, self.model_config.hidden_size)

    @hidden_size.setter
    def hidden_size(self, value: int) -> None:
        self.model_config.hidden_size = value

    @property
    def img_size(self) -> int:
        return cast(int, self.model_config.img_size)

    @img_size.setter
    def img_size(self, value: int) -> None:
        self.model_config.img_size = value

    @property
    def patch_size(self) -> int:
        return cast(int, self.model_config.patch_size)

    @patch_size.setter
    def patch_size(self, value: int) -> None:
        self.model_config.patch_size = value

    @abstractmethod
    def _build_visual_encoder(self) -> PreTrainedModel:
        pass

    @abstractmethod
    def _build_hf_config(self) -> PretrainedConfig:
        pass

    @abstractmethod
    @override
    def forward(self, images: Tensor | list[Tensor]) -> Tensor | list[Tensor]:
        pass

    def verify_config(self) -> None:
        config_pairs = [
            ("hidden_size", self.get_config("hidden_size"), self.hidden_size),
            ("img_size", self.get_config("image_size"), self.img_size),
            ("patch_size", self.get_config("patch_size"), self.patch_size),
        ]

        for key, model_value, config_value in config_pairs:
            self._verify_param_match(key, model_value, config_value)

    def get_config(self, key: str) -> int | str | None:
        vision_config = getattr(self.hf_config, "vision_config", None)
        if vision_config and hasattr(vision_config, key):
            return getattr(vision_config, key)

        return getattr(self.hf_config, key, None)

    def _verify_param_match(
        self, key: str, model_value: int | str | None, config_value: int | str | None
    ) -> None:
        capitalized_key = key.capitalize()

        if model_value is None and config_value is None:
            log.warning(f"Visual Encoder: {capitalized_key} not found in config for {self.hf_name}")
        elif model_value is not None and config_value is None:
            setattr(self, key, int(model_value))
            if hasattr(self.config, key):
                setattr(self.config, key, int(model_value))
            log.info(
                f"Visual Encoder: {capitalized_key} not found in config, using hf config: {model_value}"
            )
        elif model_value is None and config_value is not None:
            log.warning(
                f"Visual Encoder: {capitalized_key} not found in hf config for {self.hf_name}"
            )
        elif model_value is not None and config_value is not None:
            if model_value != config_value:
                error_msg = f"Visual Encoder: {capitalized_key} mismatch: hf config: {model_value} != config: {config_value}"
                log.warning(error_msg)
            else:
                log.info(
                    f"Visual Encoder: {capitalized_key} verified: hf config: {model_value} == config: {config_value}"
                )



import logging
from abc import ABC, abstractmethod
from typing import override

import torch.nn as nn
from torch import Tensor

from ...config.config_schema import ConnectorConfig

log: logging.Logger = logging.getLogger(name=__name__)


class Connector(nn.Module, ABC):
    def __init__(
        self,
        config: ConnectorConfig,
        image_hidden_size: int,
        text_hidden_size: int,
    ) -> None:
        super().__init__()
        self.config: ConnectorConfig = config
        self.name: str = self.config.name

        self.image_hidden_size: int = image_hidden_size
        self.text_hidden_size: int = text_hidden_size
        self.projection_layer: nn.Module = self.build_projection_layer()

    @abstractmethod
    def _build_projection_layer(self) -> nn.Module:
        pass

    def build_projection_layer(self) -> nn.Module:
        return self._build_projection_layer()

    @abstractmethod
    def projection(self, visual_features: Tensor) -> Tensor:
        pass

    @override
    def forward(self, visual_features: Tensor):
        return self.projection(visual_features)



import logging
from typing import Any

import torch
import torch.nn as nn
import transformers

from ..models import VLMProcessor
from ..utils import conversation as conversation_lib
from .set_trainable import set_trainable_params
from .training_arguments import TrainingArguments
from .vlm_trainer import VLMTrainer

log: logging.Logger = logging.getLogger(name=__name__)


def safe_save_model_for_hf_trainer(trainer: transformers.Trainer, output_dir: str):
    """Collects the state dict and dump to disk."""

    if trainer.deepspeed:
        torch.cuda.synchronize()
        trainer.save_model(output_dir)
        return

    state_dict = trainer.model.state_dict()
    cpu_state_dict = {key: value.cpu() for key, value in state_dict.items()}
    del state_dict
    trainer._save(output_dir, state_dict=cpu_state_dict)  # pyright: ignore


def train(model: Any, training_args: TrainingArguments, data_module: Any, processor: VLMProcessor):
    log.info("Using gradient checkpointing")
    if training_args.gradient_checkpointing:

        def make_inputs_require_grad(module: nn.Module, input: Any, output: Any) -> None:
            output.requires_grad_(True)

        model.get_input_embeddings().register_forward_hook(make_inputs_require_grad)

    conversation_lib.default_conversation = conversation_lib.conv_templates[training_args.version]

    model.config.use_cache = False
    set_trainable_params(model, training_args)

    log.info("Creating trainer")
    trainer = VLMTrainer(
        model=model,
        processing_class=processor,
        args=training_args,
        **data_module,
    )

    if training_args.resume_from_checkpoint:
        log.info(f"Resuming from checkpoint: {training_args.resume_from_checkpoint}")
        trainer.train(resume_from_checkpoint=training_args.resume_from_checkpoint)
    else:
        log.info("Training from scratch")
        trainer.train()

    log.info("Saving state")
    trainer.save_state()
    model.config.use_cache = True

    log.info("Saving model")
    safe_save_model_for_hf_trainer(trainer=trainer, output_dir=training_args.output_dir)



from typing import override

import torch.nn as nn
from torch import Tensor

from ...config.config_schema import ConnectorConfig
from .base import Connector


class IdentityConnector(Connector):
    def __init__(
        self,
        config: ConnectorConfig,
        image_hidden_size: int,
        text_hidden_size: int,
    ) -> None:
        super().__init__(config, image_hidden_size, text_hidden_size)

    @override
    def _build_projection_layer(self) -> nn.Module:
        return nn.Identity()

    @override
    def projection(self, visual_features: Tensor) -> Tensor:
        return visual_features



import torch

from ..models import VLMProcessor, get_dynamic_vlm


def push_to_hub(pretrained: str, repo_name: str):
    processor = VLMProcessor.from_pretrained(
        pretrained,
    )
    VLMForCasualLM, _ = get_dynamic_vlm(pretrained)
    model: VLMForCasualLM = VLMForCasualLM.from_pretrained(
        pretrained, low_cpu_mem_usage=True, torch_dtype=torch.bfloat16
    )
    # AutoConfig.register("vlm", VLMConfig)
    # AutoProcessor.register(VLMConfig, VLMProcessor)
    # AutoModel.register(VLMConfig, VLMForCasualLM)

    processor.push_to_hub(repo_name)
    model.push_to_hub(repo_name)



from .base import Connector
from .identity import IdentityConnector
from .linear import LinearConnector
from .mlp_n_activation import MLPConnector

__all__ = ["Connector", "IdentityConnector", "LinearConnector", "MLPConnector", "connector_map"]

connector_map = {
    "identity": IdentityConnector,
    "linear": LinearConnector,
    "mlp": MLPConnector,
}



# import torch

# from vlm.models.model import VLM

# model = VLM.load_from_checkpoint(
#     "/pasteur2/u/yuhuiz/yiming/small-vlm/outputs/2025-04-12/00-33-13/checkpoints/model.pt"
# )

# model.eval()

# language_model = model.language_model
# tokenizer = language_model.tokenizer

# # text = "a b a b a b"

# # input = tokenizer(text, return_tensors="pt")
# input = {
#     "input_ids": torch.tensor([[1, 263, 18449, 734, 29872, 1134, 734, 29872, 9909, 411, 3244]]),
#     "attention_mask": torch.tensor([[1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1]]),
# }

# outputs = language_model(input_ids=input["input_ids"], attention_mask=input["attention_mask"])

# print(outputs)

# output_ids = outputs.argmax(dim=-1)

# print(output_ids)

# output_text = tokenizer.decode(output_ids[0])

# print(output_text)

# output_ids = language_model.language_model.generate(
#     input_ids=input["input_ids"],
#     attention_mask=input["attention_mask"],
#     max_new_tokens=10,
#     do_sample=False,
# )

# print(output_ids)

# print(tokenizer.decode(output_ids[0]))



from .data_arguments import DataArguments, get_data_args
from .dataset import make_supervised_data_module
from .sampler import MultiModalLengthGroupedSampler

__all__ = [
    "get_data_args",
    "DataArguments",
    "make_supervised_data_module",
    "MultiModalLengthGroupedSampler",
]



# import string
# from types import SimpleNamespace
from typing import Any

# import pandas as pd
import torch
from PIL import Image

# from transformers import StoppingCriteria
from transformers.image_processing_utils import BaseImageProcessor
from transformers.tokenization_utils import PreTrainedTokenizer

# from vlmeval.dataset import DATASET_TYPE
# from vlmeval.smp import cn_string
# from vlmeval.vlm import BaseModel

# from ..models import VLM


def expand2square(pil_img: Image.Image, background_color: tuple):
    width, height = pil_img.size
    if width == height:
        return pil_img
    elif width > height:
        result = Image.new(pil_img.mode, (width, width), background_color)
        result.paste(pil_img, (0, (width - height) // 2))
        return result
    else:
        result = Image.new(pil_img.mode, (height, height), background_color)
        result.paste(pil_img, ((height - width) // 2, 0))
        return result


def process_images(images: list, image_processor: BaseImageProcessor, model_cfg: Any):
    image_aspect_ratio = getattr(model_cfg, "image_aspect_ratio", None)
    new_images = []
    if image_aspect_ratio == "pad":
        for image in images:
            image = expand2square(image, tuple(int(x * 255) for x in image_processor.image_mean))
            image = image_processor.preprocess(image, return_tensors="pt")["pixel_values"][0]
            new_images.append(image)
    else:
        return image_processor(images, return_tensors="pt")["pixel_values"]
    if all(x.shape == new_images[0].shape for x in new_images):
        new_images = torch.stack(new_images, dim=0)
    return new_images


def tokenizer_image_token(
    prompt: str,
    tokenizer: PreTrainedTokenizer,
    image_token_index: int,
    return_tensors: str | None = None,
):
    prompt_chunks = [tokenizer(chunk).input_ids for chunk in prompt.split("<image>")]

    def insert_separator(X: list, sep: int):
        return [ele for sublist in zip(X, [sep] * len(X), strict=False) for ele in sublist][:-1]

    input_ids = []
    offset = 0
    if (
        len(prompt_chunks) > 0
        and len(prompt_chunks[0]) > 0
        and prompt_chunks[0][0] == tokenizer.bos_token_id
    ):
        offset = 1
        input_ids.append(prompt_chunks[0][0])

    for x in insert_separator(prompt_chunks, [image_token_index] * (offset + 1)):
        input_ids.extend(x[offset:])

    if return_tensors is not None:
        if return_tensors == "pt":
            return torch.tensor(input_ids, dtype=torch.long)
        raise ValueError(f"Unsupported tensor type: {return_tensors}")
    return input_ids


# class KeywordsStoppingCriteria(StoppingCriteria):
#     def __init__(self, keywords: list, tokenizer: PreTrainedTokenizer, input_ids: list):
#         self.keywords: list = keywords
#         self.keyword_ids: list = []
#         self.max_keyword_len: int = 0
#         for keyword in keywords:
#             cur_keyword_ids = tokenizer(keyword).input_ids
#             if len(cur_keyword_ids) > 1 and cur_keyword_ids[0] == tokenizer.bos_token_id:
#                 cur_keyword_ids = cur_keyword_ids[1:]
#             if len(cur_keyword_ids) > self.max_keyword_len:
#                 self.max_keyword_len = len(cur_keyword_ids)
#             self.keyword_ids.append(torch.tensor(cur_keyword_ids))
#         self.tokenizer: PreTrainedTokenizer = tokenizer
#         self.start_len: int = input_ids.shape[1]

#     def call_for_batch(self, output_ids: torch.LongTensor, **_: Any) -> bool:
#         offset = min(output_ids.shape[1] - self.start_len, self.max_keyword_len)
#         self.keyword_ids = [keyword_id.to(output_ids.device) for keyword_id in self.keyword_ids]
#         for keyword_id in self.keyword_ids:
#             truncated_output_ids = output_ids[0, -keyword_id.shape[0] :]
#             if torch.equal(truncated_output_ids, keyword_id):
#                 return True
#         outputs = self.tokenizer.batch_decode(output_ids[:, -offset:], skip_special_tokens=True)[0]
#         for keyword in self.keywords:
#             if keyword in outputs:
#                 return True
#         return False

#     @override
#     def __call__(
#         self, output_ids: torch.LongTensor, scores: torch.FloatTensor, **kwargs: Any
#     ) -> bool:
#         outputs = []
#         for i in range(output_ids.shape[0]):
#             outputs.append(self.call_for_batch(output_ids[i].unsqueeze(0), scores))
#         return all(outputs)


# class VLMGenerator(BaseModel):
#     INSTALL_REQ: bool = True
#     INTERLEAVE: bool = True

#     def __init__(self, model_path: str, **kwargs: Any):
#         super().__init__(model_path, **kwargs)
#         self.system_prompt: str = (
#             "A chat between a curious human and an artificial intelligence assistant. "
#             "The assistant gives helpful, detailed, and polite answers to the human's questions. "
#         )
#         self.stop_str: str = "</s>"
#         self.model: VLM = VLM.from_pretrained(model_path, device="cpu")

#         self.model = self.model.cuda()
#         self.tokenizer: PreTrainedTokenizer = self.model.language_model.tokenizer
#         self.image_processor: BaseImageProcessor = self.model..visual_encoder.preprocessor
#         self.conv_mode: str = "llava_v1"
#         kwargs_default = dict(
#             do_sample=False,
#             temperature=0,
#             max_new_tokens=2048,
#             top_p=None,
#             num_beams=1,
#             use_cache=True,
#         )
#         kwargs_default.update(kwargs)
#         self.kwargs: dict = kwargs_default

#     @override
#     def use_custom_prompt(self, dataset: Any):
#         assert dataset is not None
#         if DATASET_TYPE(dataset) == "MCQ":
#             return True
#         return False

#     @override
#     def build_prompt(self, line: list, dataset: Any = None):
#         assert self.use_custom_prompt(dataset)
#         assert dataset is None or isinstance(dataset, str)
#         tgt_path = self.dump_image(line, dataset)

#         question = line["question"]
#         hint = line["hint"] if ("hint" in line and not pd.isna(line["hint"])) else None
#         if hint is not None:
#             question = hint + "\n" + question

#         options = {
#             cand: line[cand]
#             for cand in string.ascii_uppercase
#             if cand in line and not pd.isna(line[cand])
#         }
#         for key, item in options.items():
#             question += f"\n{key}. {item}"
#         prompt = question

#         if len(options):
#             prompt += (
#                 "\n请直接回答选项字母。"
#                 if cn_string(prompt)
#                 else "\nAnswer with the option's letter from the given choices directly."
#             )
#         else:
#             prompt += (
#                 "\n请直接回答问题。" if cn_string(prompt) else "\nAnswer the question directly."
#             )

#         message = [dict(type="image", value=s) for s in tgt_path]
#         message.append(dict(type="text", value=prompt))
#         return message

#     def concat_tilist(self, message: list):
#         text, images = "", []
#         for item in message:
#             if item["type"] == "text":
#                 text += item["value"]
#             elif item["type"] == "image":
#                 text += " <image> "
#                 images.append(item["value"])
#         return text, images

#     def chat_inner(self, message: list, _: Any = None):
#         image_token_index: int = self.model.model_config.language_model.image_token_index

#         prompt = self.system_prompt
#         images = []
#         for utter in message:
#             prompt += "USER: " if utter["role"] == "user" else "ASSISTANT: "
#             content, images_sub = self.concat_tilist(utter["content"])
#             prompt += content
#             images.extend(images_sub)
#             prompt += " " if utter["role"] == "user" else self.stop_str
#         assert message[-1]["role"] == "user", message
#         prompt += "ASSISTANT: "

#         images = [Image.open(s).convert("RGB") for s in images]
#         args = SimpleNamespace(image_aspect_ratio="pad")
#         image_tensor = process_images(images, self.image_processor, args).to(
#             "cuda", dtype=torch.float16
#         )

#         input_ids = (
#             tokenizer_image_token(prompt, self.tokenizer, image_token_index, return_tensors="pt")
#             .unsqueeze(0)
#             .cuda()
#         )
#         keywords = [self.stop_str]
#         stopping_criteria = KeywordsStoppingCriteria(keywords, self.tokenizer, input_ids)
#         with torch.inference_mode():
#             output_ids = self.model.generate(
#                 input_ids,
#                 images=image_tensor,
#                 stopping_criteria=[stopping_criteria],
#                 **self.kwargs,
#             )
#         output = self.tokenizer.batch_decode(output_ids, skip_special_tokens=True)[0].strip()
#         return output

#     @override
#     def generate_inner(self, message: list, _: Any = None):
#         image_token_index: int = self.model.model_config.language_model.image_token_index

#         # Support interleave text and image
#         content, images = self.concat_tilist(message)

#         images = [Image.open(s).convert("RGB") for s in images]
#         args = SimpleNamespace(image_aspect_ratio="pad")
#         if images:
#             image_tensor = process_images(images, self.image_processor, args).to(
#                 "cuda", dtype=torch.float16
#             )
#         else:
#             image_tensor = None

#         prompt = self.system_prompt + "USER: " + content + " ASSISTANT: "

#         input_ids = (
#             tokenizer_image_token(prompt, self.tokenizer, image_token_index, return_tensors="pt")
#             .unsqueeze(0)
#             .cuda()
#         )
#         keywords = [self.stop_str]
#         stopping_criteria = KeywordsStoppingCriteria(keywords, self.tokenizer, input_ids)
#         with torch.inference_mode():
#             output_ids = self.model.generate(
#                 input_ids,
#                 images=image_tensor,
#                 stopping_criteria=[stopping_criteria],
#                 **self.kwargs,
#             )

#         output = self.tokenizer.batch_decode(output_ids, skip_special_tokens=True)[0].strip()
#         return output



import logging
from typing import cast, override

from torch import Tensor
from transformers import AutoConfig, AutoModel, PretrainedConfig, PreTrainedModel

from ...config.config_schema import VisualEncoderConfig
from .base import VisualEncoder

log: logging.Logger = logging.getLogger(name=__name__)


class HFVisualEncoder(VisualEncoder):
    def __init__(self, config: VisualEncoderConfig) -> None:
        super().__init__(config)

    @override
    def _build_visual_encoder(self) -> PreTrainedModel:
        visual_encoder: PreTrainedModel = cast(
            PreTrainedModel,
            AutoModel.from_pretrained(
                self.hf_name,
                trust_remote_code=True,
            ),
        )
        if getattr(visual_encoder, "vision_model", None):
            visual_encoder = visual_encoder.vision_model  # pyright: ignore
        return visual_encoder

    @override
    def _build_hf_config(self) -> PretrainedConfig:
        return cast(
            PretrainedConfig, AutoConfig.from_pretrained(self.hf_name, trust_remote_code=True)
        )

    @override
    def forward(self, images: Tensor | list[Tensor]) -> Tensor | list[Tensor]:
        if type(images) is list:
            image_features: list[Tensor] | Tensor = []
            for image in images:
                outputs = self.visual_encoder(
                    image.unsqueeze(0),
                    output_hidden_states=True,
                )
                hidden_states: Tensor = outputs.hidden_states[self.output_layer].to(image.dtype)
                if not self.config.use_cls_token:
                    image_features.append(hidden_states[:, 1:])
                else:
                    image_features.append(hidden_states)
        else:
            outputs = self.visual_encoder(
                images,
                output_hidden_states=True,
            )
            hidden_states = outputs.hidden_states[self.output_layer].to(images.dtype)
            if not self.config.use_cls_token:
                image_features = hidden_states[:, 1:]
            else:
                image_features = hidden_states

        return image_features



import logging
import re
from typing import Any, override

import torch.nn as nn
from torch import Tensor

from ...config.config_schema import ConnectorConfig
from .base import Connector

log = logging.getLogger(__name__)


class MLPConnector(Connector):
    ACTIVATION_MAP: dict[str, type[nn.Module]] = {
        "relu": nn.ReLU,
        "gelu": nn.GELU,
        "silu": nn.SiLU,
        "tanh": nn.Tanh,
        "sigmoid": nn.Sigmoid,
    }

    @override
    def __init__(
        self,
        config: ConnectorConfig,
        image_hidden_size: int,
        text_hidden_size: int,
    ) -> None:
        self.num_layers: int = 2
        self.activation_name: str = "gelu"

        self._parse_config_name(config.name)

        super().__init__(config, image_hidden_size, text_hidden_size)

    def _parse_config_name(self, name: str) -> None:
        pattern = r"mlp_(\d+)_(\w+)"
        match = re.match(pattern, name)

        if match:
            self.num_layers = int(match.group(1))
            self.activation_name = match.group(2)

    @override
    def _build_projection_layer(self) -> nn.Module:
        if self.num_layers < 1:
            raise ValueError(f"Number of layers must be at least 1, got {self.num_layers}")

        activation_class = self.ACTIVATION_MAP.get(self.activation_name)
        if activation_class is None:
            log.error(
                f"Unsupported activation: {self.activation_name}.\nSupported activations: {list(self.ACTIVATION_MAP.keys())}"
            )
            raise ValueError(
                f"Unsupported activation: {self.activation_name}.\nSupported activations: {list(self.ACTIVATION_MAP.keys())}"
            )

        layers: list[Any] = []
        for i in range(self.num_layers):
            if i == 0:
                layers.append(nn.Linear(self.image_hidden_size, self.text_hidden_size))
            else:
                layers.append(nn.Linear(self.text_hidden_size, self.text_hidden_size))

            if i < self.num_layers - 1:
                layers.append(activation_class())

        return nn.Sequential(*layers)

    @override
    def projection(self, visual_features: Tensor) -> Tensor:
        return self.projection_layer(visual_features)



from dataclasses import dataclass, field

from hydra.core.config_store import ConfigStore
from omegaconf import MISSING  # pyright: ignore


@dataclass
class VisualEncoderConfig:
    name: str = MISSING
    hf_name: str = MISSING
    type: str = "hf_visual_encoder"
    hidden_size: int | None = None
    img_size: int | None = None
    patch_size: int | None = None
    output_layer: int | None = None
    use_cls_token: bool = False


@dataclass
class LanguageModelConfig:
    name: str = MISSING
    hf_name: str = MISSING
    type: str = "hf_language_model"
    max_seq_length: int | None = None
    image_token: str = "<image>"
    use_start_end_tokens: bool = False
    use_image_patch_token: bool = False
    image_start_token: str = "<im_start>"
    image_end_token: str = "<im_end>"
    image_patch_token: str = "<im_patch>"
    system_token: str | None = None
    user_token: str | None = None
    assistant_token: str = MISSING
    ignore_index: int = -100
    image_token_index: int = -200
    padding_side: str = "left"


@dataclass
class ConnectorConfig:
    name: str = MISSING
    type: str = MISSING


@dataclass
class ModelConfig:
    name: str = MISSING
    visual_encoder: VisualEncoderConfig = field(default_factory=VisualEncoderConfig)
    language_model: LanguageModelConfig = field(default_factory=LanguageModelConfig)
    connector: ConnectorConfig = field(default_factory=ConnectorConfig)


@dataclass
class DatasetConfig:
    name: str = MISSING
    path: str = MISSING
    type: str = "json"
    lazy_preprocess: bool = True
    is_multimodal: bool = True
    image_folder: str = MISSING
    image_aspect_ratio: str = "square"


@dataclass
class UnfreezeConfig:
    train_vision_model: bool = True
    train_language_model: bool = True
    train_connector: bool = True


@dataclass
class LearningRateConfig:
    visual_encoder_learning_rate: float = 1e-4
    language_model_learning_rate: float = 1e-4
    connector_learning_rate: float = 1e-4
    default_lr: float = 1e-4


@dataclass
class WeightDecayConfig:
    visual_encoder_weight_decay: float = 0.0
    language_model_weight_decay: float = 0.0
    connector_weight_decay: float = 0.0
    default_wd: float = 0.0


@dataclass
class TrainerConfig:
    output_dir: str = "."
    unfreeze: UnfreezeConfig = field(default_factory=UnfreezeConfig)
    learning_rate: LearningRateConfig = field(default_factory=LearningRateConfig)
    weight_decay: WeightDecayConfig = field(default_factory=WeightDecayConfig)
    per_device_train_batch_size: int = 16
    per_device_eval_batch_size: int = 4
    bf16: bool = False
    fp16: bool = False
    tf32: bool = False
    deepspeed: str | None = None
    num_train_epochs: int = 1
    save_strategy: str = "steps"
    save_steps: int = 5000
    save_total_limit: int = 20
    logging_steps: int = 1
    warmup_ratio: float = 0.0
    lr_scheduler_type: str = "linear"
    gradient_accumulation_steps: int = 1
    report_to: str | None = None
    dataloader_num_workers: int = 0
    version: str = "v0"
    group_by_modality_length: bool = False
    gradient_checkpointing: bool = False
    run_name: str = "small-vlm"
    resume_from_checkpoint: str | None = None
    from_pretrained: str | None = None
    seed: int = 42
    attn_implementation: str | None = "flash_attention_2"


@dataclass
class ModeConfig:
    is_training: bool = True


@dataclass
class InferenceConfig:
    checkpoint_path: str = MISSING
    num_inference_samples: int | None = None
    chat_template: str = "plain"


@dataclass
class AppConfig:
    mode: ModeConfig = field(default_factory=ModeConfig)
    model: ModelConfig = field(default_factory=ModelConfig)
    dataset: DatasetConfig = field(default_factory=DatasetConfig)
    trainer: TrainerConfig = field(default_factory=TrainerConfig)
    inference: InferenceConfig = field(default_factory=InferenceConfig)


def register_configs() -> None:
    cs: ConfigStore = ConfigStore.instance()
    cs.store(name="cfg", node=AppConfig)



from .config_schema import (
    AppConfig,
    ConnectorConfig,
    DatasetConfig,
    LanguageModelConfig,
    ModelConfig,
    TrainerConfig,
    VisualEncoderConfig,
    register_configs,
)

__all__ = [
    "AppConfig",
    "ModelConfig",
    "TrainerConfig",
    "register_configs",
    "DatasetConfig",
    "ConnectorConfig",
    "LanguageModelConfig",
    "VisualEncoderConfig",
]



from .inference import process_images, tokenizer_image_token
from .utils import conv_templates, push_to_hub

__all__ = [
    "process_images",
    "tokenizer_image_token",
    "conv_templates",
    "push_to_hub",
]
